package de.hft.stuttgart.citydoctor2.zip;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class CityGmlZipEntryFile implements AutoCloseable {

    private final ZipFile zip;
    private final ZipEntry zipEntry;
    private final CityGmlZipEntry cgmlZipEntry;
    private boolean closed = false;

    public CityGmlZipEntryFile(CityGmlZipEntry entry) throws IOException {
        CityGmlZipArchive archive = entry.getArchive();
        zip = new ZipFile(archive.getArchivePath().toFile());
        zipEntry = zip.getEntry(entry.getFullFileName());
        this.cgmlZipEntry = entry;
    }

    public InputStream getInputStream() throws IOException {
        if (closed){
            throw new IOException("Stream closed");
        }
        return zip.getInputStream(zipEntry);
    }

    public long getFileSize() throws IOException {
        if (closed){
            throw new IOException("Stream closed");
        }
        if (cgmlZipEntry.getFileSize() == -1L) {
            if (zipEntry.getSize() == -1) {
                long bytes = 0;
                InputStream is = new BufferedInputStream(this.getInputStream());
                for (int i = is.read(); i != -1; i = is.read()) {
                    bytes++;
                }
                cgmlZipEntry.setFileSize(bytes);
            } else {
                cgmlZipEntry.setFileSize(zipEntry.getSize());
            }
        }
        return cgmlZipEntry.getFileSize();
    }

    @Override
    public void close() throws IOException {
        zip.close();
        closed = true;
    }
}
